import setuptools

setuptools.setup(
  name="pcClustering",
  version="0.1.0",
  author="Howard Van",
  author_email="2447721464@qq.com",
  description="A simple example package for point clustering",
  long_description_content_type="text/markdown",
  url="https://github.com/pypa/sampleproject",
  packages=setuptools.find_packages(),
  classifiers=[
  "Programming Language :: Python :: 3",
  "License :: OSI Approved :: MIT License",
  "Operating System :: OS Independent",
  ],
)